package io.github.iainvm.go.players;

import io.github.iainvm.go.game.GameState;

import java.util.Scanner;

public class ConsolePlayer implements Player{
    //ConsolePlayer is a human player for the game of Go where the human enters the co-ordinates they want to place
    //their counter on the board through command line.

    private Scanner scn;
    private int pID;

    public ConsolePlayer(int pID){
        scn = new Scanner(System.in);
        this.pID = pID;
    }

    //Retrieves the move from the command line user through text input
    public int getMove(GameState gs){

        render(gs);

        //System.out.println(Arrays.toString(gs.board));

        System.out.println("Player " + pID + " make your move (0 to pass):");
        int x = getUserInput("X: ", 0, GameState.size);
        if(x == 0){
            return -1;
        }

        int y = getUserInput("y: ", 0, GameState.size);

        x--;
        y--;

        return ((y*GameState.size)+x);
    }

    private int getUserInput(String message, int bottom, int top){
        int val;
        System.out.println(message);
        val = scn.nextInt();

        while((val < bottom)||(val > top)){
            val = scn.nextInt();
        }

        return val;
    }

    //Renders the game on the command line through rudimentary text drawing
    private static void render(GameState gs){
        char col = '║';
        char row = '═';

        //Calc dimensions of rendering
        int size = (4*GameState.size)+1;

        //Make String Builder
        StringBuilder sb = new StringBuilder();

        //Render Blank Grid
        for(int j = 0; j < size; j++){
            if(((j+2)%4) == 0){
                //Draw Rows
                for(int i = 0; i < size; i++){
                    if((i+2)%4 == 0){
                        sb.append(" ");
                    }else{
                        sb.append(row);
                    }
                }
            }else{
                //Draw Columns
                for(int i = 0; i < size; i++){
                    if((i+2)%4 == 0){
                        sb.append(col);
                    }else{
                        sb.append(" ");
                    }
                }
            }
            sb.append("\n");
        }

        //Render Counters
        int x, y;
        char count, lib;

        //Iterate over board
        for(int i = 0; i < gs.board.length; i++){

            //If there is a counter
            if(gs.board[i] != 0){

                //Set counter colour
                if(gs.board[i] > 0){
                    //Black Counter
                    count = '░';
                }else{
                    //White Counter
                    count = '█';
                }

                //Get x and y co-ords of counter
                x = (4*(i%GameState.size))+2;
                y = (4*(i/GameState.size))+2;

                for(int dy = -1; dy < 2; dy++) {
                    for (int dx = -1; dx < 2; dx++) {
                        sb.setCharAt(((size+1)*(y+dy)+(x+dx)),count);
                    }
                }

                lib = Integer.toString(Math.abs(gs.board[i])).charAt(0);

                sb.setCharAt(((size+1)*y+x),lib);
            }

        }
        System.out.println(sb.toString());

    }
}