package io.github.iainvm.go.players.ai;

import io.github.iainvm.go.game.GameLogic;
import matlabcontrol.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class MCTNN extends AI{

    private MatLabContainer mat;
    private int pID;

    //Constructor
    public MCTNN(int pID){
        super(pID);
        this.pID = pID;
        try {
            mat = new MatLabContainer();
        } catch (MatlabConnectionException e) {
            e.printStackTrace();
        }
    }

    //Chose which children to add to the tree
    @Override
    public void expandChildren(TreeNode tn){

        double [][] dboard = new double[81][1];

        for(int i = 0; i < 81; i++){
            dboard[i][0] = tn.gs.board[i];
        }

        mat.setupVariable(dboard);

        double[] board = mat.runNN(pID);
        ArrayList<Integer> legalMoves = GameLogic.getLegalMoves(tn.gs);


        for(int move: top(board, legalMoves)){
            if(legalMoves.contains(move)){
                if(!tn.children.containsKey(move)){
                    tn.children.put(move, new TreeNode(tn, GameLogic.makeMove(tn.gs, move)));
                }
            }
        }


        int test = -1;

        if(legalMoves.contains(test)){
            if(!tn.children.containsKey(test)){
                tn.children.put(test, new TreeNode(tn, GameLogic.makeMove(tn.gs, test)));
            }
        }
    }

    private List<Integer> top(double[] board, ArrayList<Integer> legalMoves){
        if(legalMoves.size() == 1){
            return legalMoves;
        }
        if(legalMoves.size() <= (81*2)/3){
            return legalMoves;
        }
        List<Integer> top = new ArrayList<>();
        List<Double> medianArr = new ArrayList<>();

        for(Integer i: legalMoves){
            if(i != -1){
                medianArr.add(board[i]);
            }
        }

        double threshhold = medianArr.get((legalMoves.size()-1)/2);
//        double threshhold = 0.01;

        for(Integer i: legalMoves){
            if((i != -1)&&(board[i] >= threshhold)){
                top.add(i);
            }
        }

        return top;
    }

    public void disconnect(){
        mat.disconMatLab();
    }
}