package io.github.iainvm.go.players.ai;

import io.github.iainvm.go.game.GameLogic;
import io.github.iainvm.go.game.GameState;

import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Map;
import java.util.Random;

public class Trainer extends AI{

    private TreeNode root, current;
    private int pID, counter;
    private boolean threadRunning;
    private int simAmount = 100;

    //Constructor
    public Trainer(int pID){
        super(pID);
        this.pID=pID;
    }

    //Decide a move from a given state
    public int getMove(GameState gs){
        //Add the given gs to the tree if it's not already there

        //If the gamestate isn't the root (already added) and doesn't already exist as a child
        if(root == null){
            root = new TreeNode(null, new GameState(gs));
            current = root;
        }else{
            current.children.put(gs.lastMove, new TreeNode(current, new GameState(gs)));
            //Make new node current node
            current = current.children.get(gs.lastMove);
        }

        run();

        //exports the board along with
        saveCurrentToCSV();


        System.out.println("Player " + pID + " made " + counter + " Loops");
        System.out.println("Current probability " + current.wins/current.plays);

        //Select best move
        return selectMove();
    }

    private void saveCurrentToCSV() {
        try {
            FileWriter fwin = new FileWriter("C:\\Users\\IainV\\IdeaProjects\\GoGame\\" + pID + "in.csv", true);
            FileWriter fwou = new FileWriter("C:\\Users\\IainV\\IdeaProjects\\GoGame\\" + pID + "out.csv", true);

            for (int i = 0; i < 81; i++) {
                fwin.append(normalise(current.gs.board[i]));
                if ((i+1) % 9 != 0) {
                    fwin.append(',');
                } else {
                    fwin.append('\n');
                }
            }
            fwin.append(";");

            int move;
            for (int i = 0; i < 81; i++) {

                move = selectMove();

                if(move != i){
                    fwou.append("0");
                }else{
                    fwou.append("1");
                }

                if ((i+1) % 9 != 0) {
                    fwou.append(',');
                } else {
                    fwou.append('\n');
                }
            }
            fwou.append(";");

            fwin.flush();
            fwou.flush();
            fwin.close();
            fwou.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public String normalise(int i){
        if(i == 0){
            return "0";
        }
        if(i > 0){
            return "1";
        }
        if(i < 0){
            return "-1";
        }
        return "0";
    }

    //Picks the best move from the children of the current Node
    private int selectMove(){
        return selectBest(current).gs.lastMove;
    }

    //Picks the best move from the children of the current Node
    private static TreeNode selectBest(TreeNode tn){
        Random rnd = new Random();
        double bestScore = 0;
        double score;
        ArrayList<TreeNode> nodes = new ArrayList<>();

        for(Map.Entry<Integer, TreeNode> child : tn.children.entrySet()){
            score = (child.getValue().wins)/(child.getValue().plays);

            if(score > bestScore){
                bestScore = score;
                nodes = new ArrayList<>();
                nodes.add(child.getValue());
            }
            if(score == bestScore){
                nodes.add(child.getValue());
            }
        }

        return nodes.get(rnd.nextInt(nodes.size()));
    }

    //Create many children of current node to simulate
    @Override
    public void expandChildren(TreeNode tn){
        //Get legal moves
        ArrayList<Integer> legalMoves = GameLogic.getLegalMoves(tn.gs);
        Random rnd = new Random();

        int move, amount;

        amount = legalMoves.size();
//        amount = 1;

        for(int i = 0; i < amount; i++){
//            move = legalMoves.get(rnd.nextInt(legalMoves.size()));
            move = legalMoves.get(i);

            if(!tn.children.containsKey(move)){
                tn.children.put(move, new TreeNode(tn, GameLogic.makeMove(tn.gs, move)));
            }
        }

//        //Add a random child
//        move = legalMoves.get(rnd.nextInt(legalMoves.size()));
//        if(!tn.children.containsKey(move)){
//            tn.children.put(move, new TreeNode(tn, GameLogic.makeMove(tn.gs, move)));
//        }
    }

    //Is it a terminal node
    private static boolean isTerminal(TreeNode tn){
        //TODO: Improve
        //Problem: Game was randomly selecting 2 passes, so causing
        return GameLogic.isFinished(tn.gs);
//        return GameLogic.getLegalMoves(tn.gs).size() == 0;
    }

    //Simulate from the given node
    private void simulate(TreeNode pointer){
        Random rnd = new Random();
        GameState gs = new GameState(pointer.gs);
        ArrayList<Integer> legalMoves;
        int move;

        //While it's not a terminal node
        while((!GameLogic.isFinished(gs))&&(threadRunning)){
            //Pick a random child
            legalMoves = GameLogic.getLegalMoves(gs);
            move = legalMoves.get(rnd.nextInt(legalMoves.size()));

            gs = GameLogic.makeMove(gs, move);
        }

        //Score
        int[] score = GameLogic.score(gs);
        int playerIndex = (current.gs.currentPlayer > 0)? 0 : 1;
        boolean won = score[playerIndex] > score[(playerIndex+1)%2];
        pointer.plays++;
        if(won){
            pointer.wins++;
        }

        //Iterate back up the tree
        while(pointer.parent != null){
            pointer = pointer.parent;
            pointer.plays++;
            if(won){
                pointer.wins++;
            }
        }
    }

    @Override
    //Expands the tree from the current TreeNode
    public void run() {
        threadRunning = true;
        counter = 0;

        while(threadRunning){
            //From current node
            //Select optimal child nodes down to leaf node
            counter++;

            //If L isn't terminal (isFinished) create child
            if(!isTerminal(current)){
                expandChildren(current);
            }

            //Simulate child node
            for(Map.Entry<Integer, TreeNode> child : current.children.entrySet()){
                for(int i = 0; i < simAmount; i++) {
                    simulate(child.getValue());
                }
            }

            threadRunning = !threadRunning;

        }
    }
}