package io.github.iainvm.go.server;

import io.github.iainvm.go.game.GameLogic;
import io.github.iainvm.go.game.GameState;
import io.github.iainvm.go.players.Player;
import io.github.iainvm.go.players.ai.MCT;
import io.github.iainvm.go.players.ai.MCTNN;
import io.github.iainvm.go.players.ai.Trainer;

import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

public class Server {
    //This is the game server which implements the GameLogic and stores the current GameState of the game being played

    private GameState gs;
    private Player[] players;

    public static void main(String[] args) {
        //Create Server Object
        Server srv = new Server();
        //Start Server
        srv.start();
    }

    private Server(){
        //Set up starting game state
        GameState.size = 9;
        gs = new GameState();

        //Set up players
        players = new Player[2];
        players[0] = new MCT(1);
        players[1] = new MCT(2);
//        players[0] = new Trainer(1);
//        players[1] = new Trainer(2);
    }

    //Start the game server
    private void start(){
        boolean running = true;
        int move;

        //Start the game and loop while it's not finished
        while(running){

            int player = (gs.currentPlayer == 1)? 0 : 1;

            //Get the move of a current player
            move = players[player].getMove(gs);

            //Legality check
            while(!GameLogic.isLegal(gs, move)){
                System.out.println(((gs.currentPlayer > 0)? "Black" : "White") + "tried the illegal move of: " + move);
                //Move is not legal, remake move (should only happen for humans)
                move = players[player].getMove(gs);
            }

            //Make the legal move
            System.out.println(((gs.currentPlayer > 0)? "Black" : "White") + " is making move: " + move);
            gs = GameLogic.makeMove(gs, move);
            render(gs);

            //Check if the game has finished
            running = !GameLogic.isFinished(gs);
        }

        System.out.println("Game has finished, this is the final board");
        render(gs);
        int[] score = GameLogic.score(gs);
        System.out.println("Score:");
        System.out.println("Black: " + score[0]);
        System.out.println("White: " + score[1]);


        try {
            FileWriter fw = new FileWriter("C:\\Users\\IainV\\IdeaProjects\\GoGame\\out\\production\\GoGame\\resultsMvNN.csv", true);

            if(score[0] == score[1]){
                fw.append("draw");
            }
            if(score[0] > score[1]){
                fw.append("black");
            }
            if(score[0] < score[1]){
                fw.append("white");
            }

            fw.append("    " + score[0] + ":" + score[1] + "\n");
            fw.close();
        } catch (IOException e) {
            e.printStackTrace();
        }

        try{
            System.exit(0);
        }catch(Exception e){

        }
    }

    //Renders the game on the command line through rudimentary text drawing
    private static void render(GameState gs){
        char col = '║';
        char row = '═';

        ArrayList<Integer> legal = GameLogic.getLegalMoves(gs);

        //Calc dimensions of rendering
        int size = (4*GameState.size)+1;

        //Make String Builder
        StringBuilder sb = new StringBuilder();

        //Render Blank Grid
        for(int j = 0; j < size; j++){
            if(((j+2)%4) == 0){
                //Draw Rows
                for(int i = 0; i < size; i++){
                    if((i+2)%4 == 0){
                        sb.append(" ");
                    }else{
                        sb.append(row);
                    }
                }
            }else{
                //Draw Columns
                for(int i = 0; i < size; i++){
                    if((i+2)%4 == 0){
                        sb.append(col);
                    }else{
                        sb.append(" ");
                    }
                }
            }
            sb.append("\n");
        }

        //Render Counters
        int x, y;
        char count, lib;

        //Iterate over board
        for(int i = 0; i < gs.board.length; i++){

            //If there is a counter
            if(gs.board[i] != 0){

                //Set counter colour
                if(gs.board[i] > 0){
                    //Black Counter
                    count = '░';
                }else{
                    //White Counter
                    count = '█';
                }

                //Get x and y co-ords of counter
                x = (4*(i%GameState.size))+2;
                y = (4*(i/GameState.size))+2;

                for(int dy = -1; dy < 2; dy++) {
                    for (int dx = -1; dx < 2; dx++) {
                        sb.setCharAt(((size+1)*(y+dy)+(x+dx)),count);
                    }
                }

//                lib = Integer.toString(Math.abs(gs.board[i])).charAt(0);

                lib = Integer.toHexString(Math.abs(gs.board[i])).charAt(0);

                if(legal.contains(i)){
                    lib = 'L';
                }


                sb.setCharAt(((size+1)*y+x),lib);
            }else{

                if(legal.contains(i)){
                    lib = 'L';
                }else{
                    lib = ' ';
                }

                x = (4*(i%GameState.size))+2;
                y = (4*(i/GameState.size))+2;

                sb.setCharAt(((size+1)*y+x),lib);
            }

        }
        System.out.println(sb.toString());

    }
}